from supervisely.app import DataJson, StateJson
from supervisely.app.widgets import Widget


class InputNumber(Widget):
    def __init__(
        self,
        value: int = 1,
        min: int = 1,
        max: int = 100,
        step: int = 1,
        size: str = "small",
        controls: bool = True,
        debounce: int = 300,
        widget_id: str = None,
    ):
        self._value = value
        self._min = min
        self._max = max
        self._step = step
        self._size = size
        self._controls = controls
        self._debounce = debounce

        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        return {
            "min": self._min,
            "max": self._max,
            "step": self._step,
            "size": self._size,
            "controls": self._controls,
            "debounce": self._debounce,
        }

    def get_json_state(self):
        return {"value": self._value}

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value
        StateJson()[self.widget_id]["value"] = self._value
        StateJson().send_changes()

    def get_value(self):
        return StateJson()[self.widget_id]["value"]

    @property
    def min(self):
        return self._min

    @min.setter
    def min(self, value):
        self._min = value
        DataJson()[self.widget_id]["min"] = self._min
        DataJson().send_changes()

    @property
    def max(self):
        return self._max

    @max.setter
    def max(self, value):
        self._max = value
        DataJson()[self.widget_id]["max"] = self._max
        DataJson().send_changes()



import os
import numpy
import functools
from typing import Tuple, List, Optional
from supervisely.sly_logger import logger
from supervisely.io.fs import silent_remove
from supervisely.geometry.bitmap import Bitmap
from supervisely.imaging import image as sly_image
from supervisely.geometry.rectangle import Rectangle
from supervisely._utils import rand_str as sly_rand_str
from supervisely.annotation.annotation import Annotation
from supervisely.annotation.label import Label
from supervisely.annotation.tag import Tag
from supervisely.geometry.point_location import PointLocation
from supervisely.geometry.sliding_windows_fuzzy import (
    SlidingWindowsFuzzy,
    SlidingWindowBorderStrategy,
)


def _process_image_path(image_path: str, rect: Rectangle) -> Tuple[str, Tuple[int, int]]:
    image = sly_image.read(image_path)
    image_size = image.shape[:2]
    image_base_dir = os.path.dirname(image_path)
    image_name, image_ext = os.path.splitext(os.path.basename(image_path))

    image_crop = sly_image.crop(image, rect)
    image_crop_path = os.path.join(
        image_base_dir, sly_rand_str(10) + "_" + image_name + "_crop" + image_ext
    )
    sly_image.write(image_crop_path, image_crop)
    return image_crop_path, image_size

def _scale_ann_to_original_size(
    ann: Annotation, original_size: Tuple[int, int], rect: Rectangle
) -> Annotation:
    updated_labels = []
    for label in ann.labels:
        if type(label.geometry) is Rectangle:
            updated_geometry = Rectangle(
                top=label.geometry.top + rect.top,
                left=label.geometry.left + rect.left,
                bottom=label.geometry.bottom + rect.top,
                right=label.geometry.right + rect.left,
            )

        if type(label.geometry) is Bitmap:
            bitmap_data = label.geometry.data
            bitmap_origin = PointLocation(
                label.geometry.origin.row + rect.top,
                label.geometry.origin.col + rect.left,
            )

            updated_geometry = Bitmap(data=bitmap_data, origin=bitmap_origin)
        updated_labels.append(label.clone(geometry=updated_geometry))

    ann = ann.clone(img_size=original_size, labels=updated_labels)
    return ann

def _apply_agnostic_nms(labels: List[Label], iou_thres: Optional[float] = 0.5) -> List[Label]:
    import torch
    import torchvision
    # TODO: where we can get iou_th and conf_th?
    boxes = []
    scores = []
    for label in labels:
        label: Label
        label_rect: Rectangle = label.geometry.to_bbox()
        boxes.append([
            float(label_rect.left), 
            float(label_rect.top), 
            float(label_rect.right), 
            float(label_rect.bottom)
        ])
        conf_score: Tag = label.tags.get("confidence", None)
        if conf_score is None:
            raise ValueError("Label don't have confidence score tag named 'confidence'.")
        scores.append(conf_score.value)
    boxes = torch.tensor(boxes)
    scores = torch.tensor(scores)
    saved_inds = torchvision.ops.nms(boxes, scores, iou_thres)
    saved_labels = []
    for ind in saved_inds:
        saved_labels.append(labels[ind])
    return saved_labels


def process_image_roi(func):
    """
    Decorator for processing annotation labels before and after inference.

    Crops input image before inference if kwargs['state']['rectangle_crop'] provided
    and then scales annotation back to original image size.

    Keyword arguments:

    :param image_np: Image in numpy.ndarray format (use image_path or image_np, not both)
    :type image_np: numpy.ndarray
    :param image_path: Path to image (use image_path or image_np, not both)
    :type image_path: str
    :raises: :class:`ValueError`, if image_np or image_path invalid or not provided
    :return: Annotation in json format
    :rtype: :class:`dict`
    """
    @functools.wraps(func)
    def wrapper_inference(*args, **kwargs):
        settings = kwargs["settings"]
        rectangle_json = settings.get("rectangle")

        if rectangle_json is None:
            ann = func(*args, **kwargs)
            return ann

        rectangle = Rectangle.from_json(rectangle_json)
        if "image_np" in kwargs.keys():
            image_np = kwargs["image_np"]
            if not isinstance(image_np, numpy.ndarray):
                raise ValueError("Invalid input. Image path must be numpy.ndarray")
            original_image_size = image_np.shape[:2]
            image_crop_np = sly_image.crop(image_np, rectangle)
            kwargs["image_np"] = image_crop_np
            ann = func(*args, **kwargs)
            ann = _scale_ann_to_original_size(
                ann, original_image_size, rectangle
            )
        elif "image_path" in kwargs.keys():
            image_path = kwargs["image_path"]
            if not isinstance(image_path, str):
                raise ValueError("Invalid input. Image path must be str")
            image_crop_path, original_image_size = _process_image_path(image_path, rectangle)
            kwargs["image_path"] = image_crop_path
            ann = func(*args, **kwargs)
            ann = _scale_ann_to_original_size(
                ann, original_image_size, rectangle
            )
            silent_remove(image_crop_path)
        else:
            raise ValueError("image_np or image_path not provided!")

        return ann

    return wrapper_inference


def process_image_sliding_window(func):
    @functools.wraps(func)
    def wrapper_inference(*args, **kwargs):
        settings = kwargs["settings"]
        data_to_return = kwargs["data_to_return"]
        inference_mode = settings.get("inference_mode", "full_image")
        if inference_mode != "sliding_window":
            ann = func(*args, **kwargs)
            return ann
        sliding_window_mode = settings.get("sliding_window_mode", "basic")
        if sliding_window_mode == "none":
            ann = func(*args, **kwargs)
            return ann

        sliding_window_params = settings["sliding_window_params"]
        image_path = kwargs["image_path"]
        img = sly_image.read(image_path)
        img_h, img_w = img.shape[:2]
        windowHeight = sliding_window_params.get("windowHeight", img_h)
        windowWidth = sliding_window_params.get("windowWidth", img_w)
        overlapY = sliding_window_params.get("overlapY", 0)
        overlapX = sliding_window_params.get("overlapX", 0)
        borderStrategy = sliding_window_params.get("borderStrategy", "shift_window")

        slider = SlidingWindowsFuzzy(
            [windowHeight, windowWidth], [overlapY, overlapX], borderStrategy
        )
        rectangles = []
        for window in slider.get(img.shape[:2]):
            rectangles.append(window)

        data_to_return["slides"] = []
        all_labels = []
        for rect in rectangles:
            image_crop_path, original_image_size = _process_image_path(image_path, rect)
            kwargs["image_path"] = image_crop_path
            slice_ann: Annotation = func(*args, **kwargs)
            slice_ann = _scale_ann_to_original_size(
                slice_ann, original_image_size, rect
            )
            data_to_return["slides"].append(
                {
                    "rectangle": rect.to_json(),
                    "labels": [l.to_json() for l in slice_ann.labels],
                }
            )
            all_labels.extend(slice_ann.labels)

        all_json_labels = []
        for slide in data_to_return["slides"]:
            all_json_labels.extend(slide["labels"])

        full_rect = Rectangle(0, 0, img_h, img_w)
        all_labels_slide = {
            "rectangle": full_rect.to_json(),
            "labels": all_json_labels
        }
        data_to_return["slides"].append(all_labels_slide) # for visualization
        ann = Annotation.from_img_path(image_path)

        if sliding_window_mode == "advanced":
            labels_after_nms = _apply_agnostic_nms(all_labels)
            ann = ann.add_labels(labels_after_nms)
            all_labels_after_nms_slide = {
                "rectangle": full_rect.to_json(),
                "labels": [l.to_json() for l in labels_after_nms]
            }
            data_to_return["slides"].append(all_labels_after_nms_slide)
        else:
            ann = ann.add_labels(all_labels)
            data_to_return["slides"].append(all_labels_slide)
        return ann

    return wrapper_inference



from typing import List
from supervisely.app.jinja2 import create_env
from supervisely.app.content import DataJson, StateJson
from supervisely.app.widgets import Widget


class RadioTable(Widget):
    def __init__(
        self,
        columns: List[str],
        rows: List[List[str]],
        subtitles: dict = {},  # col_name -> subtitle
        column_formatters: dict = {},
        widget_id: str = None,
    ):

        self.columns = columns
        self._rows = rows
        self.subtitles = subtitles
        self.column_formatters = column_formatters

        self._header = []
        for col in self.columns:
            self._header.append({"title": col, "subtitle": self.subtitles.get(col)})

        self._frows = []

        super().__init__(widget_id=widget_id, file_path=__file__)

        self.rows = rows

    def get_json_data(self):
        return {
            "header": self._header,
            "frows": self._frows,
            "raw_rows_data": self.rows,
        }

    def get_json_state(self):
        return {"selectedRow": 0}

    def format_value(self, column_name: str, value):
        fn = self.column_formatters.get(column_name, self.default_formatter)
        return fn(f"data.{self.widget_id}.raw_rows_data[params.ridx][params.vidx]")

    def default_formatter(self, value):
        if value is None:
            return "-"
        return "<div> {{{{ data.{}.raw_rows_data[params.ridx][params.vidx] }}}} </div>".format(
            self.widget_id
        )

    def _update_frows(self):
        self._frows = []
        for idx, row in enumerate(self._rows):
            if len(row) != len(self.columns):
                raise ValueError(
                    f"Row #{idx} length is {len(row)} != number of columns ({len(self.columns)})"
                )
            frow = []
            for col, val in zip(self.columns, row):
                frow.append(self.format_value(col, val))
            self._frows.append(frow)

    def get_selected_row(self, state):
        widget_actual_state = state[self.widget_id]
        widget_actual_data = DataJson()[self.widget_id]
        if widget_actual_state is not None and widget_actual_data is not None:
            selected_row_index = widget_actual_state["selectedRow"]
            return self.rows[selected_row_index]

    @property
    def rows(self):
        return self._rows

    @rows.setter
    def rows(self, value):
        self._rows = value
        self._update_frows()
        DataJson()[self.widget_id]["frows"] = self._frows
        DataJson()[self.widget_id]["raw_rows_data"] = self._rows



# coding: utf-8

import uuid
from supervisely.annotation.tag import Tag, TagJsonFields
from supervisely._utils import take_with_default
from supervisely.volume_annotation.constants import KEY, ID
from supervisely.video_annotation.key_id_map import KeyIdMap


class VolumeTag(Tag):
    def __init__(
        self,
        meta,
        value=None,
        key=None,
        sly_id=None,
        labeler_login=None,
        updated_at=None,
        created_at=None,
    ):
        super(VolumeTag, self).__init__(
            meta,
            value=value,
            sly_id=sly_id,
            labeler_login=labeler_login,
            updated_at=updated_at,
            created_at=created_at,
        )
        self._key = take_with_default(key, uuid.uuid4())

    def key(self):
        return self._key

    def to_json(self, key_id_map: KeyIdMap = None):
        data_json = super(VolumeTag, self).to_json()
        if type(data_json) is str:
            # @TODO: case when tag has no value, super.to_json() returns tag name
            data_json = {TagJsonFields.TAG_NAME: data_json}
        data_json[KEY] = self.key().hex

        if key_id_map is not None:
            item_id = key_id_map.get_tag_id(self.key())
            if item_id is not None:
                data_json[ID] = item_id

        return data_json

    @classmethod
    def from_json(cls, data, tag_meta_collection, key_id_map: KeyIdMap = None):
        temp = super(VolumeTag, cls).from_json(data, tag_meta_collection)
        key = uuid.UUID(data[KEY]) if KEY in data else uuid.uuid4()

        if key_id_map is not None:
            key_id_map.add_tag(key, data.get(ID, None))

        return cls(
            meta=temp.meta,
            value=temp.value,
            key=key,
            sly_id=temp.sly_id,
            labeler_login=temp.labeler_login,
            updated_at=temp.updated_at,
            created_at=temp.created_at,
        )

    def clone(
        self,
        meta=None,
        value=None,
        key=None,
        sly_id=None,
        labeler_login=None,
        updated_at=None,
        created_at=None,
    ):
        return VolumeTag(
            meta=take_with_default(meta, self.meta),
            value=take_with_default(value, self.value),
            key=take_with_default(key, self.key),
            sly_id=take_with_default(sly_id, self.sly_id),
            labeler_login=take_with_default(labeler_login, self.labeler_login),
            updated_at=take_with_default(updated_at, self.updated_at),
            created_at=take_with_default(created_at, self.created_at),
        )



from __future__ import annotations
from typing import List
from supervisely.app.widgets import Widget
from supervisely.api.project_api import ProjectInfo
from supervisely.project.project import Project
import supervisely.imaging.color as sly_color

"""
items = [
    Select.Item(label="CPU", value="cpu"),
    Select.Item(label="GPU 0", value="cuda:0"),
    Select.Item(value="option3"),
]
r = Select(items=items, filterable=True, placeholder="select me")

f0 = Field(r, "title0")
f1 = Field(r, "title1", "description1")
f2 = Field(r, "title2", "description2", title_url="/a/b")
f3 = Field(r, "title3", "description3", description_url="/a/b")
f4 = Field(r, "title4", "description4", title_url="/a/b", description_url="/a/b")
f5 = Field(r, "title5", "with icon", icon=Field.Icon(zmdi_class="zmdi zmdi-bike"))
f6 = Field(
    r,
    "title6",
    "with image",
    icon=Field.Icon(image_url="https://i.imgur.com/0E8d8bB.png"),
)

fields = Container([f0, f1, f2, f3, f4, f5, f6])

"""


class Field(Widget):
    class Icon:
        def __init__(
            self,
            zmdi_class=None,
            color_rgb: List[int, int, int] = None,
            bg_color_rgb: List[int, int, int] = None,
            image_url=None,
        ) -> Field.Icon:
            if zmdi_class is None and image_url is None:
                raise ValueError(
                    "One of the arguments has to be defined: zmdi_class or image_url"
                )
            if zmdi_class is not None and image_url is not None:
                raise ValueError(
                    "Only one of the arguments has to be defined: zmdi_class or image_url"
                )
            if image_url is not None and (
                color_rgb is not None or bg_color_rgb is not None
            ):
                raise ValueError(
                    "Arguments color_rgb / bg_color_rgb can not be used with image_url at the same time"
                )

            if image_url is None and color_rgb is None:
                color_rgb = [255, 255, 255]

            if image_url is None and bg_color_rgb is None:
                bg_color_rgb = [0, 154, 255]

            self._zmdi_class = zmdi_class
            self._color = color_rgb
            self._bg_color = bg_color_rgb
            self._image_url = image_url
            if self._color is not None:
                sly_color._validate_color(self._color)
            if self._bg_color is not None:
                sly_color._validate_color(self._bg_color)

        def to_json(self):
            res = {}
            if self._zmdi_class is not None:
                res["className"] = self._zmdi_class
                res["color"] = sly_color.rgb2hex(self._color)
                res["bgColor"] = sly_color.rgb2hex(self._bg_color)
            if self._image_url is not None:
                res["imageUrl"] = self._image_url
            return res

    def __init__(
        self,
        content: Widget,
        title: str,
        description: str = None,
        title_url: str = None,
        description_url: str = None,
        icon: Field.Icon = None,
        widget_id: str = None,
    ):
        self._title = title
        self._description = description
        self._title_url = title_url
        self._description_url = description_url
        self._icon = icon
        self._content = content
        if self._title_url is not None and self._title is None:
            raise ValueError(
                "Title can not be specified only as url without text value"
            )
        if self._description_url is not None and self._description is None:
            raise ValueError(
                "Description can not be specified only as url without text value"
            )

        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        res = {
            "title": self._title,
            "description": self._description,
            "title_url": self._title_url,
            "description_url": self._description_url,
            "icon": None,
        }
        if self._icon is not None:
            res["icon"] = self._icon.to_json()
        return res

    def get_json_state(self):
        return None



import os
import requests

from setuptools import find_packages, setup

# @TODO: change manifest location


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as fin:
        return fin.read()


response = requests.get("https://api.github.com/repos/supervisely/supervisely/releases/latest")
version = response.json()["tag_name"]

# Dependencies do not include PyTorch, so
# supervisely_lib.nn.hosted.pytorch will not work out of the box.
# If you need to invoke that part of the code, it is very likely you
# already have PyTorch installed.
setup(
    name="supervisely",
    version=version,
    python_requires=">=3.7.1",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(
        include=["supervisely_lib", "supervisely_lib.*", "supervisely", "supervisely.*"]
    ),
    description="Supervisely Python SDK.",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/supervisely/supervisely",
    package_data={
        "": ["*.html", "*.css"],
        "supervisely": ["video/*.sh", "app/development/*.sh", "imaging/colors.json.gz"],
    },
    install_requires=[
        "numpy>=1.19, <2.0.0",
        "opencv-python>=4.5.5.62, <5.0.0.0",
        "PTable>=0.9.2, <1.0.0",
        "pillow>=5.4.1, <10.0.0",
        "protobuf>=3.14.0, <=3.20.3",
        "python-json-logger>=0.1.11, <3.0.0",
        "requests>=2.27.1, <3.0.0",
        "requests-toolbelt>=0.9.1, <1.0.0",
        "Shapely>=1.7.1, <2.0.0",
        "bidict>=0.21.2, <1.0.0",
        "varname>=0.8.1, <1.0.0",
        "python-dotenv>=0.19.2, <1.0.0",
        "pynrrd>=0.4.2, <1.0.0",
        "imgaug>=0.4.0, <1.0.0",
        "SimpleITK>=2.1.1.2, <3.0.0.0",
        "pydicom>=2.3.0, <3.0.0",
        "stringcase>=1.2.0, <2.0.0",
        "python-magic>=0.4.25, <1.0.0",
        "trimesh>=3.11.2, <4.0.0",
        "scikit-video>=1.1.11, <2.0.0",
        "uvicorn[standard]>=0.18.2, <1.0.0",
        "fastapi>=0.79.0, <1.0.0",
        "websockets>=10.3, <11.0",
        "jinja2>=3.0.3, <4.0.0",
        "psutil>=5.9.0, <6.0.0",
        "jsonpatch>=1.32, <2.0",
        "MarkupSafe>=2.1.1, <3.0.0",
        "arel>=0.2.0, <1.0.0",
        "tqdm>=4.62.3, <5.0.0",
        "pandas>=1.1.3, <=1.5.2",  # For compatibility with Python3.7
        "async_asgi_testclient",
        "PyYAML",
        "distinctipy",
        "beautifulsoup4",
        "numerize",
        "ffmpeg-python==0.2.0",
        "python-multipart==0.0.5",
    ],
    extras_require={
        "extras": [
            "docker>=5.0.3, <6.0.0",
            "imagecorruptions>=1.1.2, <2.0.0",
            "scikit-image>=0.17.1, <1.0.0",
            "matplotlib>=3.3.2, <4.0.0",
            "pascal-voc-writer>=0.1.4, <1.0.0",
            "scipy>=1.5.2, <2.0.0",
            "sk-video>=1.1.10, <2.0.0",
            "pandas>=1.1.3, <1.4.0",
        ],
        "apps": [
            "uvicorn[standard]>=0.18.2, <1.0.0",
            "fastapi>=0.79.0, <1.0.0",
            "websockets>=10.3, <11.0",
            "jinja2>=3.0.3, <4.0.0",
            "psutil>=5.9.0, <6.0.0",
            "jsonpatch>=1.32, <2.0",
            "MarkupSafe>=2.1.1, <3.0.0",
            "arel>=0.2.0, <1.0.0",
            "tqdm>=4.62.3, <5.0.0",
            "pandas>=1.1.3, <1.4.0",
        ],
        "docs": [
            "sphinx==4.4.0",
            "jinja2==3.0.3",
            "sphinx-immaterial==0.4.0",
            "sphinx-copybutton==0.4.0",
            "sphinx-autodoc-typehints==1.15.3",
            "sphinxcontrib-details-directive==0.1.0",
            "myst-parser==0.18.0",
        ],
        "sdk-no-usages": [
            "grpcio>=1.34.1, <2.0.0",
            "plotly>=4.11.0, <6.0.0",
            "psutil>=5.4.5, <6.0.0",
        ],
        # legacy dependencies
        "plugins": [
            "jsonschema>=2.6.0,<3.0.0",
        ],
        "sdk-nn-plugins": [
            "flask-restful>=0.3.7, <1.0.0",
            "Werkzeug>=1.0.1, <3.0.0",
        ],
    },
)



try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from supervisely.app import StateJson, DataJson
from supervisely.app.widgets import Widget
from supervisely.api.api import Api
from supervisely.project.project_type import ProjectType


class DestinationProject(Widget):
    class Routes:
        VALUE_CHANGED = "value_changed"

    def __init__(
        self,
        workspace_id: int,
        project_type: Literal[
            ProjectType.IMAGES,
            ProjectType.VIDEOS,
            ProjectType.VOLUMES,
            ProjectType.POINT_CLOUDS,
            ProjectType.POINT_CLOUD_EPISODES,
        ] = ProjectType.IMAGES,
        widget_id: str = None,
    ):
        self._api = Api()

        self._project_mode = "new_project"
        self._dataset_mode = "new_dataset"

        self._project_id = None
        self._dataset_id = None

        self._project_name = ""
        self._dataset_name = ""

        self._workspace_id = workspace_id
        self._project_type = project_type
        self._changes_handled = False

        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        return None

    def get_json_state(self):
        return {
            "workspace_id": str(self._workspace_id),
            "project_mode": self._project_mode,
            "project_id": self._project_id,
            "project_name": self._project_name,
            "project_type": self._project_type,
            "dataset_mode": self._dataset_mode,
            "dataset_id": self._dataset_id,
            "dataset_name": self._dataset_name,
        }

    def get_selected_project_id(self):
        return StateJson()[self.widget_id]["project_id"]

    def get_selected_dataset_id(self):
        project_id = StateJson()[self.widget_id]["project_id"]
        dataset_mode = StateJson()[self.widget_id]["dataset_mode"]
        if project_id is not None and dataset_mode == "existing_dataset":
            ds_name = StateJson()[self.widget_id]["dataset_id"]
            ds = self._api.dataset.get_info_by_name(parent_id=project_id, name=ds_name)
            return ds.id
        return None

    def get_project_name(self):
        return StateJson()[self.widget_id]["project_name"]

    def get_dataset_name(self):
        return StateJson()[self.widget_id]["dataset_name"]

    def value_changed(self, func):
        route_path = self.get_route_path(DestinationProject.Routes.VALUE_CHANGED)
        server = self._sly_app.get_server()
        self._changes_handled = True

        @server.post(route_path)
        def _click():
            value = self.get_selected_dataset_id()
            if value == "":
                value = None
            func(value)

        return _click
